/*
 * HorzBar.java
 */
package UTEStatsViewer;
import com_ibm_us_bolmar.GraphicPlotter.*;
import java.awt.*;
import java.awt.event.*;
/**
 * A horizontal bar of fixed height (in terms of user coordinates).
 */
class HorzBar extends Graphic {
   /** User coordinate height of the horizontal bars. */
   static final int HEIGHT = 1;

   /**
    * User coordinate separation between adjacent horizontal bars.
    * Note: the amount of empty space between bars is barSep - 1;
    */
   static final int SEP = 0;

   /**
    * User coordinate gap below the lowest horizontal bar and above
    * the highest horizontal bar.
    */
   static final int OFFSET = SEP / 2;

   /**
    * User coordinate increment between the bottoms of adjacent
    * horizontal bars.
    */
   static final int INCR = HEIGHT + SEP;

   HorzBar next;  // next in linked list
   protected HorzBarCategory category;
   protected String name;
   long xpos;
   int  ypos;
   long width;


   /** Hide the default constructor. */
   protected HorzBar() {}

   /**
    * A horizontal bar is defined by the following parameters.
    * @param xpos  its x-coordinate location
    * @param ypos  its y-coordinate location
    * @param width its width
    * @param name  String to be included in messages about it
    * @param category category it is in
    */
   HorzBar(long xpos, int ypos, long width, String name,
         HorzBarCategory category) {
      this.xpos = xpos;
      this.ypos = ypos;
      this.width = width;
      this.name = name;
      this.category = category;
   }


   /** Return the bounds of this object. */
   public GraphicBoundsUC obtainBounds(GraphicBoundsUC gb) {
      gb.x = xpos;
      gb.width = width;
      gb.y = ypos;
      gb.height = HEIGHT;
      return gb;
   }


   /** Paint the graphic element. */
   public void paint(java.awt.Graphics g, int depth, int depthMax) {
      // calculate normal boundaries of the filled rectangle
      int x1Fill = 1, x2Fill = getWidth() - 2, y1Fill = 1,
            y2Fill = getHeight() - 2;
/*
if ((x2Fill >= Short.MAX_VALUE) || (y2Fill >= Short.MAX_VALUE)) {
   System.out.println("have self-clipped graphic element; width = " + (x2Fill + 1) + "; height = " + (y2Fill + 1));
}
*/

      // clip along x-dimension, if necessary
      Rectangle viewportBounds = null;
      if (x2Fill >= Short.MAX_VALUE) {
         viewportBounds = getParent().getViewportBounds();
         int xClip = viewportBounds.x - getX(); // xlate to graphics coordinates
         x1Fill = Math.max(x1Fill, xClip);
         x2Fill = Math.min(x2Fill, xClip + viewportBounds.width - 1);
      }

      // clip along y-dimension, if necessary
      if (y2Fill >= Short.MAX_VALUE) {
         if (viewportBounds == null) {
            viewportBounds = getParent().getViewportBounds();
         }
         int yClip = viewportBounds.y - getY(); // xlate to graphics coordinates
         y1Fill = Math.max(y1Fill, yClip);
         y2Fill = Math.min(y2Fill, yClip + viewportBounds.height - 1);
      }
      
      // draw a rectangle of the proper color
      if ((x2Fill >= x1Fill) && (y2Fill >= y1Fill)) {
         g.setColor(category.getSelectedCategory().getForeground());
         g.fillRect(x1Fill, y1Fill, x2Fill - x1Fill + 1, y2Fill - y1Fill + 1);
      }

      // draw a border around the rectangle
      g.setColor(Color.black);
      g.drawRect(x1Fill - 1, y1Fill - 1, x2Fill - x1Fill + 2,
            y2Fill - y1Fill + 2);
   }


   /** Respond to mouse press. */
   public void mousePressed(MouseEvent e) {
      StringBuffer buf = new StringBuffer();
      buf.append(TimeValue.toString(width));
      buf.append(' ');
      buf.append('(');
      buf.append(name);
      buf.append(' ');
      buf.append(category.getSelectedCategory().getName());
      buf.append(')');
      category.msgArea.setText(buf.toString());
   }
}

