/* VertAxisLabel.java */
package UTEStatsViewer;
import java.awt.*;
import com_ibm_us_bolmar.GraphicPlotter.*;

/**
 * A vertical axis label.
 */
public class VertAxisLabel extends AxisLabel {
   /** Create a vertical axis label with the given name. */
   public VertAxisLabel(String name) {
      setName(name);
   }


   /** Return the name as the short name to be used in prompts. */
   public String getShortName() {
      return getName();
   }


   /* Return the minimum size of the Component. */
   public Dimension getMinimumSize() {
      return getPreferredSize();
   }


   /* Return the preferred size of the Component. */
   public Dimension getPreferredSize() {
      FontMetrics fontMetrics = getFontMetrics(getFont());
      int width = 2 * fontMetrics.getMaxAdvance();
      int height = getName().length() * fontMetrics.getHeight() +
            2 * fontMetrics.getLeading();
      return new Dimension(width, height);
   }


   /* Paint the Component centered in the available space. */
   public void paint(Graphics g) {
      String name = getName();
      char chars[] = name.toCharArray();
      int nameLen = name.length();
      Dimension mySize = getSize();
      int availWidth = mySize.width;
      Font font = getFont();
      FontMetrics fontMetrics = getFontMetrics(font);
      int yDelta = fontMetrics.getHeight();
      int y = (mySize.height - nameLen * yDelta) / 2 +
            fontMetrics.getAscent();

      g.setFont(font);
      for (int i = 0; i < nameLen; i++) {
         g.drawChars(chars, i, 1,
               (availWidth - fontMetrics.charWidth(chars[i]))/2, y);
         y += yDelta;
      }
   }
}
