import java.util.Vector;



class Info
{
    double  begT;
    double  endT;
    double  lenT;
    boolean blink = false;

    public  Info() {}
}


//This class represents Rectangle( State ) Info
class StateInfo extends Info
{ 
    StateGroupLabel  groupID;        // timelines window's Y axis label

    iarray           args;           // array of integer arguments
    RecDef           stateDef;       // record display desciption

    int              level = 0;      // For nested states
    StateInfo        higher = null;  // Pointer to the next higher state
                                     // enveloping this state.

    //Constructor 
    public StateInfo() 
    { 
        super();
    }
  
    public StateInfo( final SLOG_Irec_min     irec_min,
                      final SLOG_Irec_vtrargs irec_vtrargs,
                      final SLOG_ThreadInfos  thread_infos,
                      final int               view_idx )
    {
        begT    = irec_min.starttime;
        lenT    = irec_min.duration;
        endT    = begT + lenT;
        blink   = false;
        groupID = new StateGroupLabel( irec_min.origID, 
                                       thread_infos, view_idx );
        args    = new iarray( ( (SLOG_Irec_args) irec_vtrargs.firstElement() )
                              .elems );
    }

    public StateInfo(       IrecStack         irec_stack,
                      final RecDefGroup       statedefs,
                      final SLOG_ThreadInfos  thread_infos,
                      final int               view_idx )
    {
        SLOG_Irec        irec;
        SLOG_Irec_min    irec_min;
        double           irec_begtime, irec_endtime;
        iarray           irec_args, tmp_args;
        short            irec_itype;
        int              irec_order;
        int              prev_order, beg_order, end_order;
        int              Nargs;

        RecDef           statedef;

        irec         = ( SLOG_Irec ) irec_stack.pop();
        irec_min     = irec.fix;
        irec_begtime = irec_min.starttime;
        irec_endtime = irec_begtime + irec_min.duration;
        irec_order   = irec_min.bebits.GetOrder();
        irec_itype   = irec_stack.GetIntvlType();

        begT         = irec_begtime;
        endT         = irec_endtime;
        lenT         = irec_endtime - irec_begtime;
        blink        = false;
        groupID      = new StateGroupLabel( irec.fix.origID, 
                                            thread_infos, view_idx );
        args         = new iarray( ( (SLOG_Irec_args)
                                     irec.vtrargs.firstElement() ).elems );
        end_order    = irec_order;  // The 1st from Stack is the last in time
        beg_order    = irec_order;

        while ( ! irec_stack.empty() ) {
            prev_order   = irec_order;

            irec         = ( SLOG_Irec ) irec_stack.pop();
            irec_min     = irec.fix;
            irec_begtime = irec_min.starttime;
            irec_endtime = irec_begtime + irec_min.duration;
            irec_args    = ( (SLOG_Irec_args) irec.vtrargs.firstElement() )
                           .elems;
            irec_order   = irec_min.bebits.GetOrder();

            if ( irec_order > prev_order ) {
                if ( irec_args != null ) args.append( irec_args );
            }
            else if ( irec_order == prev_order ) {
                if ( irec_args != null ) {
                    if ( irec_begtime < begT )
                        args.prepend( irec_args );
                    if ( irec_endtime > endT )
                        args.append( irec_args );
                }
            }
            else { // if ( irec_order < prev_order )
                if ( irec_args != null ) args.prepend( irec_args );
            }

            if ( irec_begtime < begT ) {
                begT = irec_begtime;
                beg_order = irec_order;
            }
            if ( irec_endtime > endT ) {
                endT = irec_endtime;
                end_order = irec_order;
            }
        }   // End of while ( ! irec_stack.empty() )

        lenT = endT - begT;

        //
        if ( beg_order != 0 ) {
            begT         = Double.NEGATIVE_INFINITY;
            statedef     = statedefs.GetRecDef( irec_itype,
                                                SLOG_Const.beg_bebits );
            if ( statedef.arg_labels != null ) {
                Nargs        = statedef.arg_labels.size(); 
                if ( Nargs > 0 ) {
                    tmp_args     = new iarray();
                    Integer itmp = new Integer( Integer.MIN_VALUE );
                    for ( int ii = 0; ii < Nargs; ii++ )
                        tmp_args.addElement( itmp );
                    args.prepend( tmp_args );
                }
            }
        }
        if ( end_order != 2 ) {
            endT         = Double.POSITIVE_INFINITY;
            statedef     = statedefs.GetRecDef( irec_itype,
                                                SLOG_Const.end_bebits );
            if ( statedef.arg_labels != null ) {
                Nargs        = statedef.arg_labels.size(); 
                if ( Nargs > 0 ) {
                    tmp_args   = new iarray();
                    Integer itmp = new Integer( Integer.MIN_VALUE );
                    for ( int ii = 0; ii < Nargs; ii++ )
                        tmp_args.addElement( itmp );
                    args.append( tmp_args );
                }
            }
        }
    }

    public void SetStateDefLink( RecDef in_statedef )
    {
        stateDef = in_statedef;
    }

    public String toString()
    {
        return ( "StateInfo=[ level=" + level
                           + ", groupID=" + groupID
                           + ", begT=" + begT
                           + ", endT=" + endT
                           + ", lenT=" + lenT
                           + ", name=" + stateDef.description
                           + ", args=" + args + " ]" );
    }
}



//This class stores information about a message
class ArrowInfo extends Info
{
    StateGroupLabel  begGroupID;     // timelines window's Y axis label (begin)
    StateGroupLabel  endGroupID;     // timelines window's Y axis label ( end )

    iarray           args;           // array of integer arguments
    RecDef           arrowDef;       // record display description
  
    //Constructor for the object representing a message.
    //origID  = Task Id of the StateGroup where the message originates
    //destID  = Task Id of the StateGroup where the message ends.
    //begtime = The time at which the message originates.
    public ArrowInfo( StateGroupLabel origID,  StateGroupLabel destID, 
                      double          begtime, double          endtime )
    {
        super ();
        begGroupID = new StateGroupLabel( origID );
        endGroupID = new StateGroupLabel( destID );
        begT       = begtime;
        endT       = endtime;
        lenT       = endT - begT;
    }
  
    public ArrowInfo( final SLOG_Irec_min     irec_min,
                      final SLOG_Irec_vtrargs irec_vtrargs,
                      final SLOG_ThreadInfos  thread_infos,
                      final int               view_idx,
                      final boolean           forward )
    {
        if ( forward ) {
            begT       = irec_min.starttime;
            lenT       = irec_min.duration;
            endT       = begT + lenT;
            begGroupID = new StateGroupLabel( irec_min.origID,
                                              thread_infos, view_idx );
            endGroupID = new StateGroupLabel( irec_min.destID,
                                              thread_infos, view_idx );
        }
        else {
            endT       = irec_min.starttime;
            lenT       = irec_min.duration;
            begT       = endT + lenT;
            begGroupID = new StateGroupLabel( irec_min.destID,
                                              thread_infos, view_idx );
            endGroupID = new StateGroupLabel( irec_min.origID,
                                              thread_infos, view_idx );
        }
        args    = new iarray( ( (SLOG_Irec_args) irec_vtrargs.firstElement() )
                              .elems );

        blink      = false;
    }

    public void SetArrowDefLink( RecDef in_arrowdef )
    {
        arrowDef = in_arrowdef;
    }

    public String toString () {
        return ("ArrowInfo=[begGroupID=" + begGroupID
                          + ", endGroupID=" + endGroupID
	                  + ", begT=" + begT
                          + ", endT=" + endT
                          + ", name=" + arrowDef.description 
                          + ", args=" + args + " ]" );
    }
} 



class MtnInfo
{ 
    double begT;
    double endT;
    int []s;
  
    //Constructor 
    public MtnInfo (int n) { s = new int [n]; }
  
    public String toString() 
    {
        return ( "MtnInfo=[ begT=" + begT
                         + ", endT=" + endT
                         + ", number=" + s.length + " ]");
    }
}
