#define MAX_THREADS 1048576
#define MAX_THREADS_PER_NODE 256

/*
 * thread candidates and non-candidates become active if seen running
 * candidates are threads listed through thread enumerating
 * NONCANDIDATE threads may be recycled while CANDIDATE threads will not.
 */
#define NONCANDIDATE -1
#define CANDIDATE -2

/*
 * bebits
 */
#define B_BEBIT 2
#define E_BEBIT 1
#define BE_BEBIT 3
#define CON_BEBIT 0

/* 
 * stack element to keep track of thread events 
 */
#define MAX_B_LENGTH 28

typedef struct _stackE {
  struct _stackE *next;      /* next stack element */
  unsigned short ltid;       /* logic thread id */
  unsigned short conbit;     /* continuation bit */
  unsigned short nid;        /* global node id */
  unsigned short rectype;    /* record type */
  int length;                /* argument length in bytes */
  char buffer[MAX_B_LENGTH]; /* argument buffer */
} stackE;

/*
 * thread entry
 * markerindex is a vector of integers which are indices for global marker names
 */
typedef struct _thread_entry {
  int taskid;                   /* MPI task ID */
  int processid;                /* AIX process ID */
  int threadid;                 /* AIX kernel thread ID */
  int nodeid;                   /* node ID */
  unsigned int selectMask:1;    /* 0=available, 1=unselected */
  int ltid:31;                  /* logic thread ID */
  int threadType;               /* MPI/User/System thread */
  stackE *stacke;               /* stack element */
  int *markerindex;             /* marker index */
} thread_entry;

/*
 * UTE interval fixed file header
 */
typedef struct _interval_fixed_header {
  int profile_version;
  int header_version;
  int num_thread_entries;   /* if num_threads == 0 --> 256 entries */
  unsigned masks:2;
  unsigned fixed_header:30;
  thread_entry logicthd[MAX_THREADS_PER_NODE];
} interval_fixed_header;

/*
 * interval fixed header has N thread entries, but NOT everyone is valid
 * interval header has M thread entries, and everyone is valid
 */
typedef struct _interval_header {
  int profile_version;
  int header_version;
  int num_thread_entries; 
  unsigned masks:2;
  unsigned fixed_header:30; 
  thread_entry *logicthd;
} interval_header;

/*
 * histogram entry
 */
typedef struct {
  int count;         /* count */
  int sec;           /* start time sec */
  int nsec;          /* start time nsec */
  double totalTime;  /* total accumulation */
  double max;        /* max */
  double min;        /* min */
} histentry;

typedef struct {
  int id;
  int count;
  histentry thdhist[MAX_THREADS_PER_NODE];
} Histogram;

